from peewee import SqliteDatabase

# TODO(andre): add support for other DB types
def _sqlite_db(cfg):
    db_path = cfg['path']
    return SqliteDatabase(db_path)

TYPE_MAPPING = dict(sqlite=_sqlite_db)

class DatabaseLoader:
    def __init__(self, cfg, database_proxy, models):
        self._cfg = cfg
        self._database_proxy = database_proxy
        self._models = models

    def load(self):
        db_type = self._cfg['type']
        if db_type not in TYPE_MAPPING:
            raise Exception(f'db type {db_type} not supported. currently supported {TYPE_MAPPING.keys()}')
        database = TYPE_MAPPING[db_type](self._cfg)
        self._database_proxy.initialize(database)
        database.connect()
        database.create_tables(self._models)
        return database



class ConfigChain:
    def __init__(self, chain):
        self._chain = chain

    def __getitem__(self, key):
        return ConfigChainSection(self._chain, key)

class ConfigChainSection:
    def __init__(self, chain, section):
        self._chain = chain
        self._section = section

    def __getitem__(self, key):
        for cfg in self._chain:
            section = _get_or_none(cfg, self._section)
            if section is not None:
                value = _get_or_none(section, key)
                if value is not None:
                    return value
        raise KeyError(f'{key} not found')

def _get_or_none(config, key):
    try:
        return config[key]
    except KeyError:
        return None



class HttpException(Exception):
    def __init__(self, message, code):
        super().__init__(message)
        self.message = message
        self.code = code

def bind(app):
    @app.errorhandler(HttpException)
    def http_exception(e):
        return {'message': e.message}, e.code
